@@ -6,14 +6,17 @@ use ckb_store::ChainStore;
 use ckb_types::{
     core::{
         self,
-        cell::{resolve_transaction, CellProvider, CellStatus, HeaderChecker},
+        cell::{
+            resolve_transaction_with_options, CellProvider, CellStatus, HeaderChecker,
+            ResolveOptions,
+        },
         error::OutPointError,
     },
     packed,
     prelude::*,
     H256,
 };
-use ckb_verification::ScriptVerifier;
+use ckb_verification::{ScriptVerifier, TxVerifyEnv};
 use jsonrpc_core::Result;
 use jsonrpc_derive::rpc;
 use std::collections::HashSet;
@@ -73,9 +76,9 @@ pub trait ExperimentRpc {
     ///         {
     ///           "capacity": "0x2540be400",
     ///           "lock": {
-    ///             "args": "0x",
     ///             "code_hash": "0x28e83a1277d48add8e72fadaa9248559e1b632bab2bd60b27955ebc4c03800a5",
-    ///             "hash_type": "data"
+    ///             "hash_type": "data",
+    ///             "args": "0x"
     ///           },
     ///           "type": null
     ///         }
@@ -244,12 +247,31 @@ impl<'a> DryRunner<'a> {
 
     pub(crate) fn run(&self, tx: packed::Transaction) -> Result<DryRunResult> {
         let snapshot: &Snapshot = &self.shared.snapshot();
-        match resolve_transaction(tx.into_view(), &mut HashSet::new(), self, self, Some(2048)) {
+        let consensus = snapshot.consensus();
+        let tip_header = snapshot.tip_header();
+        let tx_env = TxVerifyEnv::new_submit(&tip_header);
+        let resolve_opts = {
+            let proposal_window = consensus.tx_proposal_window();
+            let epoch_number = tx_env.epoch_number(proposal_window);
+            let hardfork_switch = consensus.hardfork_switch();
+            ResolveOptions::new().apply_current_features(hardfork_switch, epoch_number)
+        };
+        match resolve_transaction_with_options(
+            tx.into_view(),
+            &mut HashSet::new(),
+            self,
+            self,
+            resolve_opts,
+        ) {
             Ok(resolved) => {
-                let consensus = snapshot.consensus();
                 let max_cycles = consensus.max_block_cycles;
-                match ScriptVerifier::new(&resolved, &snapshot.as_data_provider())
-                    .verify(max_cycles)
+                match ScriptVerifier::new(
+                    &resolved,
+                    consensus,
+                    &snapshot.as_data_provider(),
+                    &tx_env,
+                )
+                .verify(max_cycles)
                 {
                     Ok(cycles) => Ok(DryRunResult {
                         cycles: cycles.into(),
